@@ -1,5 +1,4 @@
 /* reg.h
-     $Id$
 
    written by Marc Singer
    11 Nov 2004
@@ -7,19 +6,9 @@
    Copyright (C) 2004 Marc Singer
 
    This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-   USA.
+   modify it under the terms of the GNU General Public License
+   version 2 as published by the Free Software Foundation.
+   Please refer to the file debian/copyright for further details.
 
    -----------
    DESCRIPTION
@@ -27,7 +16,7 @@
 
    Macros to make register file access somewhat efficient in C.  The
    4K constant comes from the fact that ARM instructions are limited
-   to a 4K immediate offset. 
+   to a 4K immediate offset.
 
 */
 
