@@ -6,19 +6,9 @@
    Copyright (C) 2004 Marc Singer
 
    This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-   USA.
+   modify it under the terms of the GNU General Public License
+   version 2 as published by the Free Software Foundation.
+   Please refer to the file debian/copyright for further details.
 
    -----------
    DESCRIPTION
@@ -33,6 +23,16 @@
 
 /* ----- Types */
 
+#if defined (CONFIG_ENV_REGION)
+extern struct descriptor_d d_env;
+extern struct descriptor_d d_envmem;
+extern struct descriptor_d* pd_env;
+#endif
+
+#if defined (CONFIG_ENV_SIZE)
+extern char __xbss(env) g_rgbEnv[CONFIG_ENV_SIZE];
+#endif
+
 struct env_d {
   char* key;
   char* default_value;
@@ -51,7 +51,7 @@ struct env_link_1 {		/* Legacy */
 
 struct env_link {
   unsigned long magic;
-  const char apexrelease[16];
+  const char apexversion[16];
   void* env_link;		/* Align loader within flash */
   void* apex_start;
   void* apex_end;
