@@ -10,32 +10,24 @@
    -----------
 
    This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-   USA.
+   modify it under the terms of the GNU General Public License
+   version 2 as published by the Free Software Foundation.
+   Please refer to the file debian/copyright for further details.
 
 */
 
 #include <config.h>
 #include <debug_ll.h>
-#include <mach/coprocessor.h>
 #include <attributes.h>
 #include <linux/string.h>
 #include <service.h>
 #include <apex.h>
+#include <asm/cp15.h>
 #include <asm/mmu.h>
 #include <drv-mem.h>
 
+//#define TALK
+
 void* pvAlloc;			/* Address of next allocatable address */
 
 void* alloc_uncached (size_t cb, size_t alignment)
@@ -55,6 +47,10 @@ void* alloc_uncached (size_t cb, size_t alignment)
 		& ~(alignment - 1));
   pvAlloc = pv + cb;
 
+#if defined (TALK)
+  printf ("%s: <- %p, 0x%x (%u)\n", __FUNCTION__, pv, cb, cb);
+#endif
+
 #if defined (CONFIG_MMU)
   mmu_protsegment (pv, 0, 0);
 #endif
