@@ -6,19 +6,9 @@
    Copyright (C) 2005 Marc Singer
 
    This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-   USA.
+   modify it under the terms of the GNU General Public License
+   version 2 as published by the Free Software Foundation.
+   Please refer to the file debian/copyright for further details.
 
    -----------
    DESCRIPTION
@@ -50,29 +40,26 @@
      buffer is enabled by the MMU Btt bits for each page or section.
    o The Linux kernel doesn't use the write-through code for the 922.
      Not sure why.
-
+   o The TRM shows that two instructions are fetched with flat
+     addressng after enabling the MMU.  We don't do anything special
+     because our MMU tables are flat as well.
 
    XSCALE
    ------
 
-   o Uses a different method for flushing caches than ARMV4.
+   o Uses a different method for cleaning cache than ARMV4.
 
 
 */
 
 #include <config.h>
-#include <mach/coprocessor.h>
 #include <attributes.h>
 #include <linux/string.h>
 #include <service.h>
 #include <apex.h>
 #include <mach/memory.h>	/* protection_for() function/macro */
 #include <debug_ll.h>
-#include "mmu.h"
-
-#if !defined (COPROCESSOR_WAIT)
-# define COPROCESSOR_WAIT
-#endif
+#include <asm/cp15.h>
 
 #if !defined (PROTECTION_FOR)
 #warning "There's not much point in enabling the MMU without declaring a PROTECTION_FOR macro."
@@ -106,7 +93,6 @@ unsigned long __xbss(ttbl) ttbl[C_PTE];
 void mmu_init (void)
 {
   int i;
-  unsigned long domain;
 
   /* Fill with 1:1 mapping sections */
   for (i = 0; i < C_PTE; ++i) {
@@ -127,20 +113,14 @@ void mmu_init (void)
       | (2<<0);			/* type(section) */
   }
 
-  domain = 0xffffffff;
-  __asm volatile ("mcr p15, 0, %0, c2, c0" : : "r" (ttbl));
-  __asm volatile ("mcr p15, 0, %0, c3, c0" : : "r" (domain));
+  STORE_TTB (ttbl);
+  STORE_DOMAIN (0xffffffff);
 
-  __asm volatile ("mcr p15, 0, %0, c7, c5, 0" : : "r" (0));  // Inv. I cache
-  __asm volatile ("mcr p15, 0, %0, c7, c6, 0" : : "r" (0));  // Inv. D cache
-  __asm volatile ("mcr p15, 0, %0, c8, c7, 0" : : "r" (0));  // Inv. TLBs
-  COPROCESSOR_WAIT;
-
-#if defined (CONFIG_CPU_XSCALE)
-		/* The XScale core guide says to do this.  It isn't
-		   clear why it should be necessary, but we oblige. */
-  __asm volatile ("mcr p15, 0, %0, c7, c10, 4" : : "r" (0)); // Drain buffer
-#endif
+  CLEANALL_DCACHE;
+  DRAIN_WRITE_BUFFER;
+  INVALIDATE_ICACHE;
+  INVALIDATE_DCACHE;
+  INVALIDATE_TLB;
 
 	/* Enable MMU */
   {
@@ -155,9 +135,10 @@ void mmu_init (void)
 		    "orr %0, %0, #(1<<12)\n\t"		/* I-cache */
 /* RR doesn't appear to make the boot faster. */
 //		    "orr %0, %0, #(1<<14)\n\t"		/* RR, predictable */
-		    "mcr p15, 0, %0, c1, c0, 0" : "=&r" (l));
+		    "mcr p15, 0, %0, c1, c0, 0\n\t"
+		    : "=&r" (l));
   }
-  COPROCESSOR_WAIT;
+  CP15_WAIT;
 }
 
 
@@ -167,6 +148,10 @@ void mmu_init (void)
    the region.  With segments defined in the MMU tables, this means
    each 1MiB defines a section.
 
+   If a region in marked as uncacheable, it is incumbent on the caller
+   to purge those rows from the cache.  We cannot do it here because
+   we don't know the range addresses that were accessed.
+
 */
 
 void mmu_protsegment (void* pv, int cacheable, int bufferable)
@@ -177,9 +162,33 @@ void mmu_protsegment (void* pv, int cacheable, int bufferable)
   ttbl[i] = (i<<20)
     | (3<<10)			/* AP(R/W) */
     | (0<<5)			/* domain(0) */
-    | (bufferable ? (1<<2) : 0)
-    | (cacheable  ? (1<<3) : 0)
+    | (bufferable ? Btt : 0)
+    | (cacheable  ? Ctt : 0)
     | (2<<0);			/* type(section) */
+
+  INVALIDATE_DTLB_VA (pv);
+  INVALIDATE_ITLB_VA (pv);
+  CP15_WAIT;
+}
+
+
+void mmu_map_segment (void* physical, void* virtual,
+                      int cacheable, int bufferable)
+{
+	/* Convert address to table index/segment # */
+  int virt = (unsigned long) (virtual) >> 20;
+  int phys = (unsigned long) (physical) >> 20;
+
+  ttbl[virt] = (phys<<20)
+    | (3<<10)			/* AP(R/W) */
+    | (0<<5)			/* domain(0) */
+    | (bufferable ? Btt : 0)
+    | (cacheable  ? Ctt : 0)
+    | (2<<0);			/* type(section) */
+
+  INVALIDATE_DTLB_VA (virtual);
+  INVALIDATE_ITLB_VA (virtual);
+  CP15_WAIT;
 }
 
 
@@ -188,14 +197,19 @@ void mmu_protsegment (void* pv, int cacheable, int bufferable)
    performs the work to switch the MMU off before transferring to
    another piece of code, Linux kernel, alternative boot loader.
 
+   *** FIXME: most of the ARM architectures do the release this way.
+   *** According to the Linux kernel, xscale does things quite
+   *** differently.  Now, it might be OK to leave this this way, but
+   *** we suspect that this may be one place where the CPU type may
+   *** require customization.
+
 */
 
 void mmu_release (void)
 {
 
-  __asm volatile ("mcr p15, 0, %0, c7, c10, 4" : : "r" (0)); // Drain buffer
-
-  CACHE_FLUSH;
+  CLEANALL_DCACHE;
+  DRAIN_WRITE_BUFFER;
 
 	/* Disable MMU */
   {
@@ -203,21 +217,25 @@ void mmu_release (void)
     __asm volatile ("mrc p15, 0, %0, c1, c0, 0\n\t"
 		    /* D-cache, Write buffer, MMU disable */
 		    "bic %0, %0, #("
-#if defined (CONFIG_HAVE_BCR) && !defined (CONFIG_FORCE_WRITHROUGH_DCACHE)
-		    "(1<<3)|"
-#endif
-		    "(1<<2)|(1<<0))\n\t"		/* D-cache, MMU-EN */
-		    "bic %0, %0, #1<<12\n\t"		/* I-cache */
+                    /* *** FIXME: it may be that we've broken xscale
+                       *** here.  The code used to clear only 2,0, and
+                       *** 3 only when BCR is set.  */
+//#if defined (CONFIG_HAVE_BCR) && !defined (CONFIG_FORCE_WRITHROUGH_DCACHE)
+//		    ""
+//#endif
+		    "(1<<3)|(1<<2)|(1<<0)"	       /* .... .... .... wc.m */
+                    ")\n\t"
+		    "bic %0, %0, #("
+                    "(1<<12)|(1<<8)"		       /* ...i ...s .... .... */
+                    ")\n\t"
 		    "mcr p15, 0, %0, c1, c0, 0" : "=&r" (l));
   }
-  COPROCESSOR_WAIT;
 
-  __asm volatile ("mcr p15, 0, %0, c7, c5, 0" : : "r" (0));  // Inv. I cache
-  __asm volatile ("mcr p15, 0, %0, c7, c6, 0" : : "r" (0));  // Inv. D cache
-  __asm volatile ("mcr p15, 0, %0, c8, c7, 0" : : "r" (0));  // Inv. TLBs
-  COPROCESSOR_WAIT;
+  INVALIDATE_CACHE;
+  INVALIDATE_TLB;
+  CP15_WAIT;
 
-  __asm volatile ("mcr p15, 0, %0, c2, c0" : : "r" (0)); /* Clear ttbl */
+  STORE_TTB (0);		/* Clear ttbl */
 }
 
 static __service_1 struct service_d mmu_service = {
