@@ -6,19 +6,9 @@
    Copyright (C) 2004 Marc Singer
 
    This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-   USA.
+   modify it under the terms of the GNU General Public License
+   version 2 as published by the Free Software Foundation.
+   Please refer to the file debian/copyright for further details.
 
    -----------
    DESCRIPTION
@@ -31,6 +21,7 @@
 #include <config.h>
 #include <asm/bootstrap.h>
 
+#if 0
 void __naked __section (.entry) entry (void)
 {
   /* Fallthrough to reset section.  Exception vectors are handled
@@ -41,3 +32,4 @@ void __naked __section (.entry) entry (void)
      case there is something it has to do.  The replacement function
      should be naked and fall-through to the .reset section. */
 }
+#endif
