@@ -6,19 +6,9 @@
    Copyright (C) 2005 Marc Singer
 
    This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-   USA.
+   modify it under the terms of the GNU General Public License
+   version 2 as published by the Free Software Foundation.
+   Please refer to the file debian/copyright for further details.
 
    -----------
    DESCRIPTION
@@ -74,6 +64,7 @@
 #define PRINTF(f...)		printf (f)
 #else
 #define PRINTF(f...)		do {} while (0)
+#define TALK 0
 #endif
 
 #if TALK > 2
@@ -335,6 +326,39 @@ static char host_mac_address[6];
 
 static int initialized;
 
+#if !defined (SMC_IO_OVERRIDE)
+
+static inline u16 SMC_inw (unsigned long base, int r)
+{
+  u16 v = *(volatile u16*) (base + r);
+  return v;
+}
+
+static inline void SMC_insw (unsigned long base, int r,
+			     void* pv, int l)
+{
+  u16* ps = (u16*) pv;
+  while (l-- > 0) {
+    *ps++ = *(volatile u16*) (base + r);
+  }
+}
+
+static inline void SMC_outw (unsigned long base, int r, u16 v)
+{
+  *(volatile u16*) (base + r) = v;
+}
+
+static inline void SMC_outsw (unsigned long base, int r,
+			      const void* p, int l)
+{
+  const unsigned short* ps = (const unsigned short*) p;
+  while (l-- > 0) {
+    *(volatile u16*) (base + r) = *ps++;
+  }
+}
+
+#endif
+
 static inline void select_bank (int bank)
 {
   write_reg (SMC_BANK, bank & 0xf);
