@@ -6,19 +6,9 @@
    Copyright (C) 2005 Marc Singer
 
    This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-   USA.
+   modify it under the terms of the GNU General Public License
+   version 2 as published by the Free Software Foundation.
+   Please refer to the file debian/copyright for further details.
 
    -----------
    DESCRIPTION
@@ -41,10 +31,25 @@ extern void (*interrupt_handlers[32])(void);
 # include "ixp42x.h"
 #endif
 
+/* *** FIXME: ifdef? */
+#include "ixp4xx-npe.h"
+
 #if defined (CONFIG_MACH_NSLU2)
 # include "nslu2.h"
 #endif
 
+#if defined (CONFIG_MACH_NAS100D)
+# include "nas100d.h"
+#endif
+
+#if defined (CONFIG_MACH_DSMG600)
+# include "dsmg600.h"
+#endif
+
+#if defined (CONFIG_MACH_FSG3)
+# include "fsg3.h"
+#endif
+
 /* ----- Types */
 
 /* ----- Globals */
