@@ -6,19 +6,9 @@
    Copyright (C) 2005 Marc Singer
 
    This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-   USA.
+   modify it under the terms of the GNU General Public License
+   version 2 as published by the Free Software Foundation.
+   Please refer to the file debian/copyright for further details.
 
    -----------
    DESCRIPTION
@@ -70,31 +60,22 @@
 #define UART_LSR_OE	 (1<<1)
 #define UART_LSR_DR	 (1<<0)
 
-extern struct driver_d* console_driver;
-
-static struct driver_d ixp42x_serial_driver;
+#define CLK		((uint32_t)(14.7456*1000*1000))
 
 void ixp42x_serial_init (void)
 {
-  u32 baudrate = 115200;
-  u32 divisor_l = 0;
-  u32 divisor_h = 0;
+  static const uint32_t baudrate = 115200;
+  uint32_t divisor;
 
   _L(LED5);
 
-  switch (baudrate) {
-  case 115200:
-    divisor_l = 8; break;
-
-  default:
-    return;
-  }
+  divisor = (CLK/baudrate)>>4;
 
   _L(LED6);
 
   UART_LCR = UART_LCR_WLS_8 | UART_LCR_STB_1 | UART_LCR_DLAB;
-  UART_DLL = divisor_l;
-  UART_DLH = divisor_h;
+  UART_DLL =  divisor       & 0xff;
+  UART_DLH = (divisor >> 8) & 0xff;
   UART_LCR = UART_LCR_WLS_8 | UART_LCR_STB_1;
   UART_FCR = UART_FCR_TRFIFOE;
 
@@ -105,10 +86,6 @@ void ixp42x_serial_init (void)
 
   _L(LED7);
 
-  if (console_driver == 0)
-    console_driver = &ixp42x_serial_driver;
-
-//  console_driver->write (0, "Console initialized\r\n", 21);
   _L(LED8);
 
 #if !defined (DISABLE_SECOND_UART_INIT)
@@ -167,6 +144,8 @@ static __driver_0 struct driver_d ixp42x_serial_driver = {
   .name = "serial-ixp42x",
   .description = "ixp42x serial driver",
   .flags = DRIVER_SERIAL | DRIVER_CONSOLE,
+  .open = open_helper,          /* Always succeed */
+  .close = close_helper,
   .read = ixp42x_serial_read,
   .write = ixp42x_serial_write,
   .poll = ixp42x_serial_poll,
