@@ -6,19 +6,9 @@
    Copyright (C) 2005 Marc Singer
 
    This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-   USA.
+   modify it under the terms of the GNU General Public License
+   version 2 as published by the Free Software Foundation.
+   Please refer to the file debian/copyright for further details.
 
    -----------
    DESCRIPTION
@@ -41,7 +31,7 @@
      CPLD JTAG chain from crashing the board.  See the target_init ()
      code. */
 
-#if defined (CONFIG_MAC_LPD7A40X)
+#if defined (CONFIG_MACH_LPD7A40X)
 # define DRV_CLCDC_SETUP\
 	({ GPIO_PINMUX |= (1<<1) | (1<<0); }) /* LCDVD[15:4] */
 #endif
@@ -62,14 +52,36 @@
 
 #endif
 
+#if defined (CONFIG_MACH_KARMA)
+# define DRV_CLCDC_SETUP\
+	({ extern void karma_clcdc_setup (void); \
+	   GPIO_PINMUX |= (1<<1) | (1<<0);\
+	    karma_clcdc_setup (); })
+
+# define DRV_CLCDC_WAKE\
+	({ extern void karma_clcdc_wake (void);\
+	   karma_clcdc_wake (); })
+
+# define DRV_CLCDC_SLEEP\
+	({ extern void karma_clcdc_sleep (void);\
+	   karma_clcdc_sleep (); })
+
+#endif
+
 #if defined (CONFIG_MACH_LPD7A400)
 # define DRV_CLCDC_BACKLIGHT_ENABLE\
 	({ CPLD_CONTROL |= CPLD_CONTROL_LCD_VEEEN; })
+# define DRV_CLCDC_BACKLIGHT_DISABLE\
+	({ CPLD_CONTROL &= ~CPLD_CONTROL_LCD_VEEEN; })
 #endif
+
 #if defined (CONFIG_MACH_LPD7A404)
 # define DRV_CLCDC_BACKLIGHT_ENABLE\
 	({ GPIO_PCDD &= ~(1<<3); GPIO_PCD  |= (1<<3); })
+# define DRV_CLCDC_BACKLIGHT_DISABLE\
+	({ GPIO_PCD  &= ~(1<<3); })
 #endif
+
 #if defined (CONFIG_MACH_COMPANION)
 
 # define BL_FREQ	(5*1000)	/* Hz */
@@ -80,17 +92,20 @@
 # define DRV_CLCDC_BACKLIGHT_ENABLE\
 	({ PWM_TC3 = PWM_TC3_V; PWM_DC3 = PWM_DC3_V; \
 	   PWM_INV3 = 0; PWM_EN3 = 1; })
+# define DRV_CLCDC_BACKLIGHT_DISABLE\
+	({ PWM_EN3 = 0; })
 #endif
 
-#if defined (CONFIG_MACH_LPD7A400)
-# define DRV_CLCDC_BACKLIGHT_DISABLE\
-	({ CPLD_CONTROL &= ~CPLD_CONTROL_LCD_VEEEN; })
-#endif
-#if defined (CONFIG_MACH_LPD7A404)
-# define DRV_CLCDC_BACKLIGHT_DISABLE\
-	({ GPIO_PCD  &= ~(1<<3); })
-#endif
-#if defined (CONFIG_MACH_COMPANION)
+#if defined (CONFIG_MACH_KARMA)
+
+# define BL_FREQ	(12*1000)	/* Hz */
+# define BL_DUTY	60		/* percentage*100 */
+# define PWM_TC3_V	((14745600/BL_FREQ) - 1)
+# define PWM_DC3_V	((BL_DUTY*(PWM_TC3_V + 1)/100) - 1)
+
+# define DRV_CLCDC_BACKLIGHT_ENABLE\
+	({ PWM_TC3 = PWM_TC3_V; PWM_DC3 = PWM_DC3_V; \
+	   PWM_INV3 = 0; PWM_EN3 = 1; })
 # define DRV_CLCDC_BACKLIGHT_DISABLE\
 	({ PWM_EN3 = 0; })
 #endif
