@@ -6,19 +6,9 @@
    Copyright (C) 2005 Marc Singer
 
    This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-   USA.
+   modify it under the terms of the GNU General Public License
+   version 2 as published by the Free Software Foundation.
+   Please refer to the file debian/copyright for further details.
 
    -----------
    DESCRIPTION
@@ -75,7 +65,7 @@
 //#define DUMP_RECEIVERS
 
 
-#if TALK > 0
+#if defined (TALK) && TALK > 0
 # define DBG(l,f...)		if (l <= TALK) printf (f);
 #else
 # define DBG(l,f...)		do {} while (0)
@@ -456,12 +446,16 @@ void udp_setup (struct ethernet_frame* frame,
 		u16 source_port, size_t cb)
 {
   const char* addr = arp_cache_lookup (destination_ip);
+  size_t cbFrame;
 //  if (!addr)
 //    addr = arp_cache_lookup (gw_ip_address);
   if (!addr)			/* *** FIXME: shouldn't be possible */
     return;
 
-  cb = (cb + 1) & ~1;
+//  cb = (cb + 1) & ~1;
+  cbFrame = (cb + 1) & ~1;
+  if (cbFrame != cb)
+    *(UDP_F (frame)->data + cb) = 0; /* zero extra byte */
 
   memset (IPV4_F (frame), 0,
 	  sizeof (struct header_ipv4) + sizeof (struct header_udp));
@@ -475,7 +469,7 @@ void udp_setup (struct ethernet_frame* frame,
   ETH_F (frame)->protocol = HTONS (ETH_PROTO_IP);
 
   /* IPv4 header */
-  IPV4_F (frame)->version_ihl = 4<<4 | 5;
+  IPV4_F (frame)->version_ihl = 4<<4 | 5; /* Version 4; Header Length 5 */
   IPV4_F (frame)->length
     = htons (sizeof (struct header_ipv4) + sizeof (struct header_udp) + cb);
   IPV4_F (frame)->ttl = 64;
@@ -512,7 +506,8 @@ void udp_setup (struct ethernet_frame* frame,
 
   frame->cb = sizeof (struct header_ethernet)
     + sizeof (struct header_ipv4)
-    + sizeof (struct header_udp) + cb;
+    + sizeof (struct header_udp)
+    + cbFrame;
 }
 
 
